#!/bin/sh -ax

. /routerbot/version.sh
. /routerbot/v$ROUTERBOT_VERSION/config.sh

export camera_pan_pos
export camera_elevation_pos
export engine_speed
export camera_speed

cmd() {
  echo "CMD> $1"
  echo "$1" >> /dev/tts/1
}

get_distance() {
  ret=`dd count=1 bs=1 if=/dev/tts/1 2>/dev/null|hexdump -e '1/1 "%u"'&`
  cmd VA
  cmd VA
  return $ret
}

init_robot() {
  stty -F /dev/tts/1 115200 cs8 -crtscts -parenb -clocal -icanon -inlcr
  set_camera_speed 100
  set_engine_speed 100
  cmd "#${ENGINE_RIGHT_SERVO}P${ENGINE_RIGHT_STOP}#${ENGINE_LEFT_SERVO}P${ENGINE_LEFT_STOP}#${CAMERA_PAN_SERVO}P${CAMERA_PAN_FWD}#${CAMERA_ELEVATION_SERVO}P${CAMERA_ELEVATION_FWD}T1000"
  camera_pan_pos=$CAMERA_PAN_FWD
  camera_elevation_pos=$CAMERA_ELEVATION_FWD
}

set_camera_speed() {
  camera_speed=$1
}

set_engine_speed() {
  engine_speed=$1
}

look() {
  cmd=""
  for arg in $*; do
    case "$arg" in
    "up")
      cmd="$cmd#${CAMERA_ELEVATION_SERVO}P${CAMERA_ELEVATION_UP}S${camera_speed}"
      camera_elevation_pos=$CAMERA_ELEVATION_UP
      ;;
    "down")
      cmd="$cmd#${CAMERA_ELEVATION_SERVO}P${CAMERA_ELEVATION_DOWN}S${camera_speed}"
      camera_elevation_pos=$CAMERA_ELEVATION_DOWN
      ;;
    "ahead")
      cmd="$cmd#${CAMERA_ELEVATION_SERVO}P${CAMERA_ELEVATION_FWD}S${camera_speed}"
      camera_elevation_pos=$CAMERA_ELEVATION_FWD
      ;;
    "back")
      cmd="$cmd#${CAMERA_ELEVATION_SERVO}P${CAMERA_ELEVATION_MIN}S${camera_speed}"
      camera_elevation_pos=$CAMERA_ELEVATION_MIN
      ;;
    "forward")
      cmd="$cmd#${CAMERA_PAN_SERVO}P${CAMERA_PAN_FWD}S${camera_speed}"
      camera_pan_pos=$CAMERA_PAN_FWD
      ;;
    "left")
      pos=$(($CAMERA_PAN_FWD-$CAMERA_PAN_DELTA))
      cmd="$cmd#${CAMERA_PAN_SERVO}P${pos}S${camera_speed}"
      camera_pan_pos=$pos
      ;;
    "right")
      pos=$(($CAMERA_PAN_FWD+$CAMERA_PAN_DELTA))
      cmd="$cmd#${CAMERA_PAN_SERVO}P${pos}S${camera_speed}"
      camera_pan_pos=$pos
      ;;
    "higher")
      pos=$(($camera_elevation_pos-$CAMERA_TURN_SPEED))
      if [ $pos -lt $CAMERA_ELEVATION_MIN ]; then
        pos=$CAMERA_ELEVATION_MIN
      fi
      camera_elevation_pos=$pos
      cmd="$cmd#${CAMERA_ELEVATION_SERVO}P${pos}S${CAMERA_TURN_SPEED}" 
      ;;
    "lower")
      pos=$(($camera_elevation_pos+$CAMERA_TURN_SPEED))
      if [ $pos -gt $CAMERA_ELEVATION_MAX ]; then
        pos=$CAMERA_ELEVATION_MAX
      fi
      camera_elevation_pos=$pos
      cmd="$cmd#${CAMERA_ELEVATION_SERVO}P${pos}S${CAMERA_TURN_SPEED}" 
      ;;
    "lefter")
      pos=$(($camera_pan_pos-$CAMERA_TURN_SPEED))
      camera_pan_min=$(($CAMERA_PAN_FWD-$CAMERA_PAN_DELTA))
      if [ $pos -lt $camera_pan_min ]; then
        pos=$camera_pan_min
      fi
      camera_pan_pos=$pos
      cmd="$cmd#${CAMERA_PAN_SERVO}P${pos}S${CAMERA_TURN_SPEED}" 
      ;;
    "righter")
      pos=$(($camera_pan_pos+$CAMERA_TURN_SPEED))
      camera_pan_max=$(($CAMERA_PAN_FWD+$CAMERA_PAN_DELTA))
      if [ $pos -gt $camera_pan_max ]; then
        pos=$camera_pan_max
      fi
      camera_pan_pos=$pos
      cmd="$cmd#${CAMERA_PAN_SERVO}P${pos}S${CAMERA_TURN_SPEED}" 
      ;;
    esac
  done
  cmd $cmd
}

lookat() {
  set -x
  pan=`expr ${CAMERA_PAN_FWD} - ${CAMERA_PAN_DELTA} + $1 \* 2 \* ${CAMERA_PAN_DELTA} / $3`
  elevation=`expr ${CAMERA_ELEVATION_MIN} + $2 \* \( ${CAMERA_ELEVATION_MAX} - ${CAMERA_ELEVATION_MIN} \) / $4`
  cmd "#${CAMERA_PAN_SERVO}P${pan}S${CAMERA_TURN_SPEED}#${CAMERA_ELEVATION_SERVO}P${elevation}S${CAMERA_TURN_SPEED}"
}

halt_camera() {
  cmd "#${CAMERA_ELEVATION_SERVO}P0#${CAMERA_PAN_SERVO}P0"
}

halt_engine() {
  cmd "#${ENGINE_RIGHT_SERVO}P0#${ENGINE_LEFT_SERVO}P0" 
}

halt() {
  halt_camera
  halt_engine
}

reset_camera() {
  cmd "#${CAMERA_PAN_SERVO}P${CAMERA_PAN_FWD}#${CAMERA_ELEVATION_SERVO}P${CAMERA_ELEVATION_FWD}"
}

fast_reset_camera() {
  cmd "#${CAMERA_PAN_SERVO}P${CAMERA_PAN_FWD}S${CAMERA_RESET_SPEED}#${CAMERA_ELEVATION_SERVO}P${CAMERA_ELEVATION_FWD}S${CAMERA_RESET_SPEED}"
}

stop() {
  cmd "#${ENGINE_RIGHT_SERVO}P${ENGINE_RIGHT_STOP}#${ENGINE_LEFT_SERVO}P${ENGINE_LEFT_STOP}"
}

go() {
  if [ $# == 2 ]; then
    set_engine_speed $2
  fi
  cmd=""
  case "$1" in
  "forward")
    lpos=$(($ENGINE_LEFT_STOP-$engine_speed))
    rpos=$(($ENGINE_RIGHT_STOP+$engine_speed))
    cmd="#${ENGINE_RIGHT_SERVO}P${rpos}#${ENGINE_LEFT_SERVO}P${lpos}"
    ;;
  "back")
    lpos=$(($ENGINE_LEFT_STOP+$engine_speed))
    rpos=$(($ENGINE_RIGHT_STOP-$engine_speed))
    cmd="#${ENGINE_RIGHT_SERVO}P${rpos}#${ENGINE_LEFT_SERVO}P${lpos}"
    ;;
  esac
  cmd $cmd
}


turn() {
  if [ $# == 2 ]; then
    set_engine_speed $2
  fi
  cmd=""
  case "$1" in
  "left90")
    lpos=$(($ENGINE_LEFT_STOP+$RIGHT_ANGLE_TURN_SPEED))
    rpos=$(($ENGINE_RIGHT_STOP+$RIGHT_ANGLE_TURN_SPEED))
    cmd "#${ENGINE_RIGHT_SERVO}P${rpos}#${ENGINE_LEFT_SERVO}P${lpos}"
    sleep 1
    stop
    ;;
  "right90")
    lpos=$(($ENGINE_LEFT_STOP-$RIGHT_ANGLE_TURN_SPEED))
    rpos=$(($ENGINE_RIGHT_STOP-$RIGHT_ANGLE_TURN_SPEED))
    cmd "#${ENGINE_RIGHT_SERVO}P${rpos}#${ENGINE_LEFT_SERVO}P${lpos}"
    sleep 1
    stop
    ;;
  "left")
    lpos=$(($ENGINE_LEFT_STOP+$engine_speed))
    rpos=$(($ENGINE_RIGHT_STOP+$engine_speed))
    cmd "#${ENGINE_RIGHT_SERVO}P${rpos}#${ENGINE_LEFT_SERVO}P${lpos}"
    sleep 1
    stop
    ;;
  "right")
    lpos=$(($ENGINE_LEFT_STOP-$engine_speed))
    rpos=$(($ENGINE_RIGHT_STOP-$engine_speed))
    cmd "#${ENGINE_RIGHT_SERVO}P${rpos}#${ENGINE_LEFT_SERVO}P${lpos}"
    sleep 1
    stop
    ;;
  "left_cont")
    lpos=$(($ENGINE_LEFT_STOP+$engine_speed))
    rpos=$(($ENGINE_RIGHT_STOP+$engine_speed))
    cmd "#${ENGINE_RIGHT_SERVO}P${rpos}#${ENGINE_LEFT_SERVO}P${lpos}"
    ;;
  "right_cont")
    lpos=$(($ENGINE_LEFT_STOP-$engine_speed))
    rpos=$(($ENGINE_RIGHT_STOP-$engine_speed))
    cmd "#${ENGINE_RIGHT_SERVO}P${rpos}#${ENGINE_LEFT_SERVO}P${lpos}"
    ;;
  "right180")
    lpos=$(($ENGINE_LEFT_STOP-$RIGHT_ANGLE_TURN_SPEED))
    rpos=$(($ENGINE_RIGHT_STOP-$RIGHT_ANGLE_TURN_SPEED))
    cmd "#${ENGINE_RIGHT_SERVO}P${rpos}#${ENGINE_LEFT_SERVO}P${lpos}"
    sleep 2
    stop
    ;;
  "left180")
    lpos=$(($ENGINE_LEFT_STOP+$RIGHT_ANGLE_TURN_SPEED))
    rpos=$(($ENGINE_RIGHT_STOP+$RIGHT_ANGLE_TURN_SPEED))
    cmd "#${ENGINE_RIGHT_SERVO}P${rpos}#${ENGINE_LEFT_SERVO}P${lpos}"
    sleep 2
    stop
    ;;
  "around")
    lpos=$(($ENGINE_LEFT_STOP+$TURN_SPEED))
    rpos=$(($ENGINE_RIGHT_STOP+$TURN_SPEED+$TURN_SPEED+$TURN_SPEED))
    cmd "#${ENGINE_RIGHT_SERVO}P${rpos}#${ENGINE_LEFT_SERVO}P${lpos}"
    sleep 10
    stop
    ;;
  esac
}

  
say() {                                                                                                                            
  cspd=$camera_speed
  set_camera_speed 600
  case "$1" in                                                                                                                      
  "yes")                                                                                                                         
    look up
    usleep 100000
    look down
    usleep 500000
    look up
    usleep 400000
    reset_camera
    ;;
  "no")
    #pos=$(($CAMERA_PAN_FWD-$CAMERA_PAN_DELTA))
    #cmd "#${CAMERA_PAN_SERVO}P${pos}S${camera_speed}"
    #sleep 1
    #pos=$(($CAMERA_PAN_FWD+$CAMERA_PAN_DELTA))
    #cmd "#${CAMERA_PAN_SERVO}P${pos}S${camera_speed}"
    look left
    usleep 300000
    look right
    usleep 600000
    look left
    usleep 300000
    reset_camera
    ;;
  esac
  set_camera_speed $cspd
}  
  
# Always init the robot first
emo() {                                                                                                                            
  cspd=$camera_speed
  espd=$engine_speed
  set_camera_speed 600
  case "$1" in                                                                                                                      
  "shy")                                                                                                                         
    look up forward
    set_camera_speed 1600
    look lower righter
    turn left
    for i in 1 2; do
      turn right
      look lower righter
      turn right
      look lower righter
      turn left
      look lower righter
      turn left  
      look lower righter
    done
    turn right
    ;;
  "excited")                                                                                                                         
    set_engine_speed 500
    turn left_cont
    usleep 10000
    for i in 1 2 3; do
      turn right_cont
      usleep 200000
      turn left_cont
      usleep 200000
    done
    turn right_cont
    usleep 10000
    stop
    ;;
  esac
  set_camera_speed $cspd
  set_engine_speed $espd
}

init_robot

export cmd go turn look emo say init stop set_camera_speed set_engine_speed halt_camera halt_engine halt
