#!/bin/sh -x

. /routerbot/v$ROUTERBOT_VERSION/bin/config.sh

export engine_speed
export camera_pan_pos
export camera_tilt_pos
export camera_speed

cmd() { # executes a command in terms of SSC-32 servo controller
  usleep 50
  echo "CMD> $1"
  #echo "$1" >> /dev/tts/1
	/routerbot/v$ROUTERBOT_VERSION/bin/command $1 0 2
}

get_distance() { # returnes the distance to the object the camera is looking at
	/routerbot/v$ROUTERBOT_VERSION/bin/command VD 1 2
}

init_robot() { # initializes robot
  stty -F /dev/tts/1 115200 cs8 -crtscts -parenb -clocal -icanon -inlcr
	rangefinder_on
  set_camera_speed 2000
  set_engine_speed 200
  cmd "#${ENGINE_RIGHT_FRONT_SERVO}P${ENGINE_RIGHT_FRONT_STOP}#${ENGINE_RIGHT_REAR_SERVO}P${ENGINE_RIGHT_REAR_STOP}#${ENGINE_LEFT_FRONT_SERVO}P${ENGINE_LEFT_FRONT_STOP}#${ENGINE_LEFT_REAR_SERVO}P${ENGINE_LEFT_REAR_STOP}#${CAMERA_PAN_SERVO}P${CAMERA_PAN_FWD}#${CAMERA_TILT_SERVO}P${CAMERA_TILT_FWD}T1000"
  camera_pan_pos=$CAMERA_PAN_FWD
  camera_tilt_pos=$CAMERA_TILT_FWD
	camera_lights_on
	say yes
	camera_lights_off
}

set_camera_speed() { # sets the camera speed
  camera_speed=$1
}

set_engine_speed() { # sets the engine speed
  engine_speed=$1
}

look() { # changes the camera position (up|down|ahead|back|forward|left|right|higher|lower|lefer|righter)
  cmd=""
  for arg in $*; do
    case "$arg" in
    "up")
      cmd="$cmd#${CAMERA_TILT_SERVO}P${CAMERA_TILT_UP}S${camera_speed}"
      camera_tilt_pos=$CAMERA_TILT_UP
      ;;
    "down")
      cmd="$cmd#${CAMERA_TILT_SERVO}P${CAMERA_TILT_DOWN}S${camera_speed}"
      camera_tilt_pos=$CAMERA_TILT_DOWN
      ;;
    "ahead")
      cmd="$cmd#${CAMERA_TILT_SERVO}P${CAMERA_TILT_FWD}S${camera_speed}"
      camera_tilt_pos=$CAMERA_TILT_FWD
      ;;
    "back")
      cmd="$cmd#${CAMERA_TILT_SERVO}P${CAMERA_TILT_MIN}S${camera_speed}"
      camera_tilt_pos=$CAMERA_TILT_MIN
      ;;
    "forward")
      cmd="$cmd#${CAMERA_PAN_SERVO}P${CAMERA_PAN_FWD}S${camera_speed}"
      camera_pan_pos=$CAMERA_PAN_FWD
      ;;
    "left")
      pos=$(($CAMERA_PAN_FWD-$CAMERA_PAN_DELTA))
      cmd="$cmd#${CAMERA_PAN_SERVO}P${pos}S${camera_speed}"
      camera_pan_pos=$pos
      ;;
    "right")
      pos=$(($CAMERA_PAN_FWD+$CAMERA_PAN_DELTA))
      cmd="$cmd#${CAMERA_PAN_SERVO}P${pos}S${camera_speed}"
      camera_pan_pos=$pos
      ;;
    "higher")
      pos=$(($camera_tilt_pos-$CAMERA_TURN_SPEED))
      if [ $pos -lt $CAMERA_TILT_MIN ]; then
        pos=$CAMERA_TILT_MIN
      fi
      camera_tilt_pos=$pos
      cmd="$cmd#${CAMERA_TILT_SERVO}P${pos}S${CAMERA_TURN_SPEED}" 
      ;;
    "lower")
      pos=$(($camera_tilt_pos+$CAMERA_TURN_SPEED))
      if [ $pos -gt $CAMERA_TILT_MAX ]; then
        pos=$CAMERA_TILT_MAX
      fi
      camera_tilt_pos=$pos
      cmd="$cmd#${CAMERA_TILT_SERVO}P${pos}S${CAMERA_TURN_SPEED}" 
      ;;
    "lefter")
      pos=$(($camera_pan_pos-$CAMERA_TURN_SPEED))
      camera_pan_min=$(($CAMERA_PAN_FWD-$CAMERA_PAN_DELTA))
      if [ $pos -lt $camera_pan_min ]; then
        pos=$camera_pan_min
      fi
      camera_pan_pos=$pos
      cmd="$cmd#${CAMERA_PAN_SERVO}P${pos}S${CAMERA_TURN_SPEED}" 
      ;;
    "righter")
      pos=$(($camera_pan_pos+$CAMERA_TURN_SPEED))
      camera_pan_max=$(($CAMERA_PAN_FWD+$CAMERA_PAN_DELTA))
      if [ $pos -gt $camera_pan_max ]; then
        pos=$camera_pan_max
      fi
      camera_pan_pos=$pos
      cmd="$cmd#${CAMERA_PAN_SERVO}P${pos}S${CAMERA_TURN_SPEED}" 
      ;;
    esac
  done
  cmd $cmd
}

lookat() { # positions the camera absolutely (lookat <X> <Y>)
  pan=`expr ${CAMERA_PAN_FWD} - ${CAMERA_PAN_DELTA} + $1 \* 2 \* ${CAMERA_PAN_DELTA} / $3`
  TILT=`expr ${CAMERA_TILT_MIN} + $2 \* \( ${CAMERA_TILT_MAX} - ${CAMERA_TILT_MIN} \) / $4`
  cmd "#${CAMERA_PAN_SERVO}P${pan}S${CAMERA_TURN_SPEED}#${CAMERA_TILT_SERVO}P${TILT}S${CAMERA_TURN_SPEED}"
}

halt_camera() { # stops the camera movement
  cmd "#${CAMERA_TILT_SERVO}P0#${CAMERA_PAN_SERVO}P0"
}

halt_engine() { # stops the engine movement
  cmd "#${ENGINE_RIGHT_FRONT_SERVO}P0#${ENGINE_LEFT_FRONT_SERVO}P0" 
  cmd "#${ENGINE_RIGHT_REAR_SERVO}P0#${ENGINE_LEFT_REAR_SERVO}P0" 
  usleep 100
}

halt() { # stops all robot movement at once
  halt_camera
  halt_engine
}

reset_camera() { # positions the camera to look forward
  cmd "#${CAMERA_PAN_SERVO}P${CAMERA_PAN_FWD}#${CAMERA_TILT_SERVO}P${CAMERA_TILT_FWD}"
}

fast_reset_camera() { # quickly moves the camera to default position
  cmd "#${CAMERA_PAN_SERVO}P${CAMERA_PAN_FWD}S${CAMERA_RESET_SPEED}#${CAMERA_TILT_SERVO}P${CAMERA_TILT_FWD}S${CAMERA_RESET_SPEED}"
}

stop() { # stops roubot movement (not camera)
  cmd "#${ENGINE_RIGHT_FRONT_SERVO}P${ENGINE_RIGHT_FRONT_STOP}#${ENGINE_RIGHT_REAR_SERVO}P${ENGINE_RIGHT_REAR_STOP}#${ENGINE_LEFT_FRONT_SERVO}P${ENGINE_LEFT_FRONT_STOP}#${ENGINE_LEFT_REAR_SERVO}P${ENGINE_LEFT_REAR_STOP}"
}

go() { # moves robot forward or backward (go forward|backward <SPEED>)
  halt_engine
  if [ $# == 2 ]; then
    set_engine_speed $2
  fi
  cmd=""
  case "$1" in
  "forward")
    lfpos=$(($ENGINE_LEFT_FRONT_STOP-$engine_speed))
    lrpos=$(($ENGINE_LEFT_REAR_STOP-$engine_speed))
    rfpos=$(($ENGINE_RIGHT_FRONT_STOP+$engine_speed))
    rrpos=$(($ENGINE_RIGHT_REAR_STOP+$engine_speed))
    cmd1="#${ENGINE_RIGHT_FRONT_SERVO}P${rfpos}"
    cmd2="#${ENGINE_RIGHT_REAR_SERVO}P${rrpos}"
    cmd3="#${ENGINE_LEFT_FRONT_SERVO}P${lfpos}"
    cmd4="#${ENGINE_LEFT_REAR_SERVO}P${lrpos}"
    ;;
  "back")
    lfpos=$(($ENGINE_LEFT_FRONT_STOP+$engine_speed))
    lrpos=$(($ENGINE_LEFT_REAR_STOP+$engine_speed))
    rfpos=$(($ENGINE_RIGHT_FRONT_STOP-$engine_speed))
    rrpos=$(($ENGINE_RIGHT_REAR_STOP-$engine_speed))
    cmd1="#${ENGINE_RIGHT_REAR_SERVO}P${rrpos}"
    cmd2="#${ENGINE_RIGHT_FRONT_SERVO}P${rfpos}#"
    cmd3="#${ENGINE_LEFT_REAR_SERVO}P${lrpos}"
    cmd4="#${ENGINE_LEFT_FRONT_SERVO}P${lfpos}"
    ;;
  esac
  cmd $cmd1
  cmd $cmd3
  cmd $cmd2
  cmd $cmd4
}

turn() { # makes the robot turn (turn left90|right90|left|right|left_cont|right_cont|left180|right180|around)
  if [ $# == 2 ]; then
    set_engine_speed $2
  fi
  cmd=""
  case "$1" in
  "left90")
    lfpos=$(($ENGINE_LEFT_FRONT_STOP+$RIGHT_ANGLE_TURN_SPEED))
    lrpos=$(($ENGINE_LEFT_REAR_STOP+$RIGHT_ANGLE_TURN_SPEED))
    rfpos=$(($ENGINE_RIGHT_FRONT_STOP+$RIGHT_ANGLE_TURN_SPEED))
    rrpos=$(($ENGINE_RIGHT_REAR_STOP+$RIGHT_ANGLE_TURN_SPEED))
    cmd1="#${ENGINE_RIGHT_FRONT_SERVO}P${rfpos}"
    cmd2="#${ENGINE_RIGHT_REAR_SERVO}P${rrpos}"
    cmd3="#${ENGINE_LEFT_FRONT_SERVO}P${lfpos}"
    cmd4="#${ENGINE_LEFT_REAR_SERVO}P${lrpos}"
		cmd $cmd1
		cmd $cmd3
		cmd $cmd2
		cmd $cmd4
    sleep 2
    stop
    ;;
  "right90")
    lfpos=$(($ENGINE_LEFT_FRONT_STOP-$RIGHT_ANGLE_TURN_SPEED))
    lrpos=$(($ENGINE_LEFT_REAR_STOP-$RIGHT_ANGLE_TURN_SPEED))
    rfpos=$(($ENGINE_RIGHT_FRONT_STOP-$RIGHT_ANGLE_TURN_SPEED))
    rrpos=$(($ENGINE_RIGHT_REAR_STOP-$RIGHT_ANGLE_TURN_SPEED))
    cmd1="#${ENGINE_RIGHT_FRONT_SERVO}P${rfpos}"
    cmd2="#${ENGINE_RIGHT_REAR_SERVO}P${rrpos}"
    cmd3="#${ENGINE_LEFT_FRONT_SERVO}P${lfpos}"
    cmd4="#${ENGINE_LEFT_REAR_SERVO}P${lrpos}"
		cmd $cmd1
		cmd $cmd3
		cmd $cmd2
		cmd $cmd4
    sleep 2
    stop
    ;;
  "left")
		turn left_cont
    usleep 100000
    stop
    ;;
  "right")
		turn right_cont
    usleep 100000
    stop
    ;;
  "left_cont")
    lfpos=$(($ENGINE_LEFT_FRONT_STOP+$engine_speed))
    lrpos=$(($ENGINE_LEFT_REAR_STOP+$engine_speed))
    rfpos=$(($ENGINE_RIGHT_FRONT_STOP+$engine_speed))
    rrpos=$(($ENGINE_RIGHT_REAR_STOP+$engine_speed))
    cmd1="#${ENGINE_RIGHT_FRONT_SERVO}P${rfpos}"
    cmd2="#${ENGINE_RIGHT_REAR_SERVO}P${rrpos}"
    cmd3="#${ENGINE_LEFT_FRONT_SERVO}P${lfpos}"
    cmd4="#${ENGINE_LEFT_REAR_SERVO}P${lrpos}"
		cmd $cmd1
		cmd $cmd3
		cmd $cmd2
		cmd $cmd4
    ;;
  "right_cont")
    lfpos=$(($ENGINE_LEFT_FRONT_STOP-$engine_speed))
    lrpos=$(($ENGINE_LEFT_REAR_STOP-$engine_speed))
    rfpos=$(($ENGINE_RIGHT_FRONT_STOP-$engine_speed))
    rrpos=$(($ENGINE_RIGHT_REAR_STOP-$engine_speed))
    cmd1="#${ENGINE_RIGHT_FRONT_SERVO}P${rfpos}"
    cmd2="#${ENGINE_RIGHT_REAR_SERVO}P${rrpos}"
    cmd3="#${ENGINE_LEFT_FRONT_SERVO}P${lfpos}"
    cmd4="#${ENGINE_LEFT_REAR_SERVO}P${lrpos}"
		cmd $cmd1
		cmd $cmd3
		cmd $cmd2
		cmd $cmd4
    ;;
  "right180")
		turn right90
		turn right90
    ;;
  "left180")
		turn left90
		turn left90
    ;;
  "around")
		turn left90
		turn left90
		turn left90
		turn left90
    ;;
  esac
}

  
say() { # make robot "say" something (say yes|no                                                                                                                          
  cspd=$camera_speed
  set_camera_speed 600
  case "$1" in                                                                                                                      
  "yes")                                                                                                                         
    look up
    usleep 100000
    look down
    usleep 500000
    look up
    usleep 400000
    reset_camera
    ;;
  "no")
    look left
    usleep 300000
    look right
    usleep 600000
    look left
    usleep 300000
    reset_camera
    ;;
  esac
  set_camera_speed $cspd
}  

on() { # sets the line high. For instance "on 15" can turn on lights connected to PWM 15  
  cmd "#$1H"                                                                                                   
}                                                                                                                   

off() { # sets the line low. For instance "off 15" can turn off lights connected to PWM 15  
  cmd "#$1L"                                                                                                   
}                                                                                                                   

camera_lights_on() { # turns on the camera headlights
  on ${LIGHTS_CAMERA}
}

camera_lights_off() { # turns off the camera headlights
  off ${LIGHTS_CAMERA}
}
  
bottom_lights_on() { # turns on the bottom lights
  on ${LIGHTS_BOTTOM}
}

bottom_lights_off() { # turns off the bottom lights
  off ${LIGHTS_BOTTOM}
}

rangefinder_on() { # turns on the range finder
  on ${RANGEFINDER_POWER}
}

rangefinder_off() { # turns off the rangefinder
  off ${RANGEFINDER_POWER}
}

poweroff_robot() { # turns off the routerbot
	poweroff
}

reboot_robot() { # reboots the routerbot
	reboot
}

# Always init the robot first
emo() { # make robot express an emotion (emo shy|excited)
  cspd=$camera_speed
  espd=$engine_speed
  set_camera_speed 600
  case "$1" in                                                                                                                      
  "shy")                                                                                                                         
    look up forward
    set_camera_speed 1600
    look lower righter
    turn left
    for i in 1 2; do
      turn right
      look lower righter
      turn right
      look lower righter
      turn left
      look lower righter
      turn left  
      look lower righter
    done
    turn right
    ;;
  "excited")                                                                                                                         
    set_engine_speed 500
    turn left_cont
    usleep 10000
    for i in 1 2 3; do
      turn right_cont
      usleep 200000
      turn left_cont
      usleep 200000
    done
    turn right_cont
    usleep 10000
    stop
    ;;
  esac
  set_camera_speed $cspd
  set_engine_speed $espd
}

reconfigure_router() {
	echo $1 $2 $3
	ssid=$2
	wepkey=$3
	case "$1" in
		"ap")
			configure_access_point $ssid $wepkey
			;;
		"sta")
			configure_standalone $ssid $wepkey
			;;
		"no_wifi")
			configure_no_wifi
			;;
		esac
	reboot
}

# the method below do network configuration

configure_access_point() {
	ssid=$1
	wepkey=$2
cat <<EOT > /etc/config/dhcp
config dhcp
        option interface        wl0
        option start    100
        option limit    50
        option leasetime        12h
        option dnsserver        192.168.1.11

config dhcp
        option interface        wan
        option ignore   1

config dhcp
        option interface        vlan2
        option start 236
        option limit 0
        option leasetime 12h
EOT
cat <<EOT > /etc/config/network
# LAN port 1 in vlan3 is used for recovery and always configured to 192.168.1.234
# LAN ports 2 and 3 in vlan0 are unused
# LAN port 4 in vlan2 is used to connect the wireless camera
# WAN port is unused
# WiFi wl0 is used as an access point

config switch eth0
        option vlan0    "1 2 5*"
        option vlan1    "4 5"
        option vlan2    "0 5"
        option vlan3    "3 5"

config interface loopback
        option ifname   "lo"
        option proto    static
        option ipaddr   127.0.0.1
        option netmask  255.0.0.0

config interface        vlan2
        option type     bridge
        option ifname   "eth0.2 wl0"
        option proto    static
        option ipaddr   192.168.1.235
        option netmask  255.255.255.0

config interface        vlan3
        option ifname   "eth0.3"
        option proto    static
        option ipaddr   192.168.1.234
        option netmask  255.255.255.0

config interface wl0
        option ifname   "wl0"
        option proto    static
        option ipaddr   192.168.1.11
        option netmask  255.255.255.0
EOT
cat <<EOT > /etc/config/wireless
config wifi-device  wl0
        option type     broadcom
        option channel  5
        option disabled '0'

config wifi-iface
        option device   wl0
        option network  vlan2
        option mode     ap
        option encryption wep
        option key      "1"
        option ssid     "$ssid"
        option key1     "$wepkey"
EOT
}

configure_standalone() {
	ssid=$1
	wepkey=$2
cat <<EOT > /etc/config/network
# LAN port 1 in vlan3 is used for recovery and always configured to 192.168.1.234
# LAN ports 2 and 3 in vlan0 are unused
# LAN port 4 in vlan2 is used to connect the wireless camera
# WAN port is unused
# WiFi wl0 is used as an access point

config switch eth0
        option vlan0    "0 1 2 4 5*"
        option vlan3    "3 5"

config interface loopback
        option ifname   "lo"
        option proto    static
        option ipaddr   127.0.0.1
        option netmask  255.0.0.0

config interface        vlan0
        option type     bridge
        option ifname   "eth0.0 wl0"
        option proto    dhcp

config interface        vlan3
        option ifname   "eth0.3"
        option proto    static
        option ipaddr   192.168.1.234
        option netmask  255.255.255.0

config interface wl0
        option ifname   "wl0"
        option proto    dhcp
EOT
cat <<EOT > /etc/config/wireless
config wifi-device  wl0
        option type     broadcom
        option channel  5
        option disabled '0'

config wifi-iface
        option device   wl0
        option network  vlan0
        option mode     sta
        option encryption wep
        option key      "1"
        option ssid     "$ssid"
        option key1     "$wepkey"
EOT
echo "# the routerbot is configured in standalone mode. Please use web interface to change that." > /etc/config/dhcp
}

configure_no_wifi() {
cat <<EOT > /etc/config/network
config switch eth0
	option vlan0    "0 1 2 3 5*"
	option vlan1    "4 5"

config interface loopback
	option ifname	"lo"
	option proto	static
	option ipaddr	127.0.0.1
	option netmask	255.0.0.0

config interface	vlan0
	option ifname	"eth0.0"
	option proto	static
	option ipaddr	192.168.1.234
	option netmask	255.255.255.0
EOT
echo "# the routerbot is configured in no_wifi mode. Please use web interface to change that." > /etc/config/wireless
echo "# the routerbot is configured in no_wifi mode. Please use web interface to change that." > /etc/config/dhcp
}

#init_robot
#export cmd go turn look emo say init_robot stop set_camera_speed set_engine_speed halt_camera halt_engine halt poweroff_robot reboot_robot
